/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */

package org.openntf.attachmentviewer;

import java.util.Dictionary;
import java.util.Hashtable;

import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.openntf.attachmentviewer.services.*;
import org.osgi.framework.BundleContext;

import com.ibm.pvc.webcontainer.listeners.HttpSettingListener;

/**
 * The activator class controls the plug-in life cycle
 */
public class Activator extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "org.openntf.attachmentviewer";

	// The shared instance
	private static Activator plugin;
	private static BundleContext bc;
	private int portNumber;
	private String hostName;
	

	/**
	 * The constructor
	 */
	public Activator() {
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		plugin = this;
		bc = context;
		
		this.getPreferenceStore().setDefault("debug", false);
		
		registerFileProviders(context);
		
		context.registerService(HttpSettingListener.class.getName(), new HttpSettingListener(){

			public void settingsAdded(String arg0, Dictionary arg1) {
				
			}

			public void settingsModified(String arg0, Dictionary settings) {
                Object iPort = settings.get( HttpSettingListener.HTTP_PORT );
                if (iPort instanceof Integer[]) {
                        portNumber = ((Integer[])iPort)[0].intValue();
                } else {
                        portNumber = ((Integer)iPort).intValue();
                }

	        String sHost = (String) 
				settings.get( HttpSettingListener.ADDRESS );
                if (HttpSettingListener.ALL_ADDRESSES.equals( sHost )) {
                        hostName = "localhost";
                } else {
                        hostName = sHost;
                }
				
			}

			public void settingsRemoved(String arg0) {
				// TODO Auto-generated method stub
				
			}
		
		}, null);
		
	}

	private void registerFileProviders(BundleContext context) {
		
		//Image service
	    Dictionary properties = new Hashtable();
	    properties.put("type", ".jpg, .gif, .bmp, .png, .ico");

		context.registerService(IAttachmentViewerProvider.class.getName(), new ImageFileProvider(), properties);
		
		//SWF Service
		properties = new Hashtable();
	    properties.put("type", ".swf");

		context.registerService(IAttachmentViewerProvider.class.getName(), new SWFFileProvider(), properties);

		//PDF Service
		properties = new Hashtable();
	    properties.put("type", ".pdf");

		context.registerService(IAttachmentViewerProvider.class.getName(), new PDFFileProvider(), properties);
		
		//Text File Services
		properties = new Hashtable();
	    properties.put("type", ".txt, .html, .xml, .cpp, .c, .java, .h, .hpp, .php, .mf, .properties, .js");
		context.registerService(IAttachmentViewerProvider.class.getName(), new TextFileProvider(), properties);
		
		//Quicktime Files
		QTFileProvider qtFP = new QTFileProvider();

		properties = new Hashtable();
	    properties.put("type", ".mp4, .avi, .mov, .m4v, .3gp, .m2v");
		context.registerService(IAttachmentViewerProvider.class.getName(), qtFP, properties);
		
		//Zip File Services
		properties = new Hashtable();
	    properties.put("type", ".zip, .jar");
		context.registerService(IAttachmentViewerProvider.class.getName(), new ZipFileProvider(), properties);

		//ODF Services
		properties = new Hashtable();
	    properties.put("type", ".odp, .odt, .docx, .dotx, .docm, .dotm, .xlsx, .xlsm, .xltx, .xltm, .pptx, .pptm, .potx");
		context.registerService(IAttachmentViewerProvider.class.getName(), new ODFProvider(), properties);
		
	}

	/*
	 * (non-Javadoc)
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		plugin = null;
		
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static Activator getDefault() {
		return plugin;
	}
	public static BundleContext getBundleContext() {
		return bc;
	}
	public int getPortNumber() {
		return portNumber;
	}

	public String getHostName() {
		return hostName;
	}	
}
