/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */
package org.openntf.attachmentviewer;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.Base64;
import org.openntf.attachmentviewer.services.IAttachmentViewerProvider;
import org.osgi.framework.BundleContext;
import org.osgi.framework.InvalidSyntaxException;
import org.osgi.framework.ServiceReference;

public class ProxyServer extends HttpServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		String uri = req.getRequestURI();
		
		if (uri.contains("extensions.js")){
			writeExtensionScripts(req, resp);			
		}
		else if (uri.contains("/content/")){
			if (writeHTMLContent(req, resp, uri) == false){
				super.doGet(req, resp);
			}
		}else{
			if (writeFile(req, resp, uri) == false){
				super.doGet(req, resp);
			}
		}
	}

	private void writeExtensionScripts(HttpServletRequest req,
			HttpServletResponse resp) throws IOException {
		
		OutputStream os = resp.getOutputStream();
		StringBuffer buffer = new StringBuffer();
		
		resp.setContentType("text/javascript");
				
		BundleContext ctx = Activator.getBundleContext();
		
		ServiceReference[] refs = null;
		
		try {
			refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), null);
		} catch (InvalidSyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		buffer.append( "var extensions = [");

		boolean addComma = false;
		
		for (int x=0; x< refs.length; x++){
			IAttachmentViewerProvider provider = (IAttachmentViewerProvider)ctx.getService(refs[x]);
			
			if (provider.hasScript()){
				
				if (addComma)
					buffer.append(",\n");
				
				String type = refs[x].getProperty("type").toString();
				buffer.append("\n{\n\ttype:\"" + type + "\", \n\tf :" + provider.getScript() + "\n}");
				
				addComma = true;
			}
			ctx.ungetService(refs[x]);
		}
		buffer.append("];\n\n");
		
		buffer.append("function getScript(type){\n");
		buffer.append("\tfor (var x=0; x<extensions.length; x++){\n");
		buffer.append("\t\tif(extensions[x].type.indexOf(type) >= 0){\n");
		buffer.append("\t\t\treturn extensions[x].f;\n");
		buffer.append("\t\t}\n");
		buffer.append("\t}\n");
		buffer.append("}\n");
		
		os.write(buffer.toString().getBytes());
		os.close();
	}

	private boolean writeHTMLContent(HttpServletRequest req,
			HttpServletResponse resp, String uri) throws IOException {
		
		int idx = uri.indexOf("?");
		String tmpName = uri;
		
		if (idx > 0)
			tmpName = uri.substring(0, idx-1);
		
		String extension = tmpName.substring(tmpName.lastIndexOf(".")+1, tmpName.length()).toLowerCase();
		
		BundleContext ctx = Activator.getBundleContext();
		
		ServiceReference[] refs = null;
		
		try {
			refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), "(type=*." + extension + "*)");
		} catch (InvalidSyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		idx = uri.lastIndexOf("/");
		String key = uri.substring(idx+1);
		key = key.substring(0, key.lastIndexOf("."));
		
		key = new String (Base64.decodeBase64(key.getBytes()));
				
		File f = new File(key);
		
		if (f.isFile() == false) {
			return false;
		}
		if (refs == null || refs.length == 0){
			return false;
		}
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();

		IAttachmentViewerProvider provider = (IAttachmentViewerProvider)ctx.getService(refs[0]);
		
		provider.writeHTML(f, buffer, 0);
				
		ServletOutputStream os = resp.getOutputStream();
		os.write(buffer.toByteArray());
		os.close();
		
		ctx.ungetService(refs[0]);
		
		return true;
	}

	private boolean writeFile(HttpServletRequest req, HttpServletResponse resp,
			String uri) throws ServletException, IOException {

		int idx = uri.indexOf("?");
		String tmpName = uri;
		
		if (idx > 0)
			tmpName = uri.substring(0, idx-1);

		String extension = tmpName.substring(tmpName.lastIndexOf(".")+1, tmpName.length()).toLowerCase();
		
		BundleContext ctx = Activator.getBundleContext();
		
		ServiceReference[] refs = null;
		
		try {
			refs = ctx.getServiceReferences(IAttachmentViewerProvider.class.getName(), "(type=*." + extension + "*)");
		} catch (InvalidSyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		idx = uri.lastIndexOf("/");
		String key = uri.substring(idx+1);
		key = key.substring(0, key.lastIndexOf("."));
		
		key = new String (Base64.decodeBase64(key.getBytes()));
				
		File f = new File(key);
		
		if (f.isFile() == false) {
			return false;
		}
		
		ServletOutputStream os = resp.getOutputStream();

		if (refs == null || refs.length == 0){
			
			FileInputStream bif;
			try {
				bif = new FileInputStream(f);
			
				byte buffer[] = new byte[1024];
				int bytesRead = 0;
				
				while ( (bytesRead = bif.read(buffer)) > 0){
					
					os.write(buffer, 0, bytesRead);
		
				}		
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			os.close();
			
			return true;
		}
		
		IAttachmentViewerProvider provider = (IAttachmentViewerProvider)ctx.getService(refs[0]);
		
		resp.setContentType(provider.getContentType());
		
		provider.writeFile(f, os);
		
		os.close();
		ctx.ungetService(refs[0]);
		
		return true;
	}

	
}
