/*
 *  Copyright IBM Corp. 2010
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at:
 * 
 * http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or 
 * implied. See the License for the specific language governing 
 * permissions and limitations under the License.
 */

package org.openntf.attachmentviewer.services;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;

import org.openntf.attachmentviewer.services.AbstractFileProvider;

public class TextFileProvider extends AbstractFileProvider {

	public String getContentType() {
		// TODO Auto-generated method stub
		return null;
	}

	public void writeFile(File f, OutputStream bos) {

		try {
			BufferedReader bif = new BufferedReader(new FileReader(f));
	
			String chunk = null;			
			StringBuffer buffer = new StringBuffer();
			
			chunk = bif.readLine();
			while (chunk != null) {									
				buffer.append(chunk);
				chunk = bif.readLine();
			}
			
			bos.write(buffer.toString().getBytes());
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	@Override
	public void writeHTML(File f, OutputStream bos, int limit) {

		String buf = getFileContents(f, limit);
		try {
			
			StringBuffer out = new StringBuffer();
			
			out.append("<pre>");
			
			if (limit > 0 && buf.length() > limit){
				out.append("<br>...<br>");
			}else{
				out.append(buf);
			}
			
			out.append("</pre>");
				
			bos.write(out.toString().getBytes());
			
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}	
	private String getFileContents(File f, int limit) {
		try {
			String name = f.getName();
			BufferedReader bif = new BufferedReader(new FileReader(f));
	
			String chunk = null;
				
			StringBuffer buffer = new StringBuffer();
						
			chunk = bif.readLine();
			while (chunk != null) {					
				
				chunk = chunk.replace(">", "&gt");
				chunk = chunk.replace("<", "&lt");
				chunk = chunk.replace("'", "&apos;");
				
				if (name.endsWith("java") ||
						name.endsWith("c")||
						name.endsWith("cpp")||
						name.endsWith("h")||
						name.endsWith("hpp")){	
					
					chunk = chunk.replace("if ", "<span style=\"color:darkred\"><b>if</b> </span>");
					chunk = chunk.replace("else ", "<span style=\"color:darkred\"><b>else</b> </span>");
					chunk = chunk.replace("public ", "<span style=\"color:darkred\"><b>public</b> </span>");
					chunk = chunk.replace("private ", "<span style=\"color:darkred\"><b>private</b> </span>");
					chunk = chunk.replace("void ", "<span style=\"color:darkred\"><b>void</b> </span>");
					chunk = chunk.replace("class ", "<span style=\"color:darkred\"><b>class</b> </span>");
					chunk = chunk.replace("import ", "<span style=\"color:darkred\"><b>import</b> </span>");
					chunk = chunk.replace("package ", "<span style=\"color:darkred\"><b>package</b> </span>");
					chunk = chunk.replace("while ", "<span style=\"color:darkred\"><b>while</b> </span>");
					chunk = chunk.replace("null ", "<span style=\"color:darkred\"><b>null</b> </span>");
					chunk = chunk.replace("new ", "<span style=\"color:darkred\"><b>new</b> </span>");
					chunk = chunk.replace("for ", "<span style=\"color:darkred\"><b>for</b> </span>");
					chunk = chunk.replace("int ", "<span style=\"color:darkred\"><b>int</b> </span>");
					chunk = chunk.replace("double ", "<span style=\"color:darkred\"><b>double</b> </span>");
					chunk = chunk.replace("float ", "<span style=\"color:darkred\"><b>float</b> </span>");
					chunk = chunk.replace("static ", "<span style=\"color:darkred\"><b>static</b> </span>");
					chunk = chunk.replace("final ", "<span style=\"color:darkred\"><b>final</b> </span>");
					chunk = chunk.replace("boolean ", "<span style=\"color:darkred\"><b>boolean</b> </span>");
	
					chunk = chunk.replace("(", "<span style=\"color:black\"><b>(</b></span>");
					chunk = chunk.replace(")", "<span style=\"color:black\"><b>)</b></span>");
					chunk = chunk.replace("{", "<span style=\"color:black\"><b>{</b></span>");
					chunk = chunk.replace("}", "<span style=\"color:black\"><b>}</b></span>");
	
				}
				
				buffer.append(chunk + "<br>");
				
				if (limit > 0 && buffer.length() > limit){
					buffer.append("<br>...<br>");
					break;
				}
				
				chunk = bif.readLine();
			}
			
			return buffer.toString();
			
			
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return null;
	}	
}
